<?php

namespace Drupal\views_dates\Plugin\views\filter;

use Drupal\views\Plugin\views\filter\InOperator;

/**
 * Filter to handle dates stored as a timestamp.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("views_dates_year_number")
 */
class YearNumber extends InOperator {

  /**
   * {@inheritdoc}
   */
  protected function opSimple() {
    if (empty($this->value)) {
      return;
    }
    $this->ensureMyTable();

    /** @var \Drupal\views\Plugin\views\query\Sql $query */
    $query = $this->query;

    $placeholder = $this->placeholder().'[]';
    // PostgreSQL can use EXTRACT (YEAR FROM field) result is numeric.
    // MySQL can use YEAR(field) result is numeric.
    // SQLite has no equivalent.
    $fieldexpr = $this->getDateField();
    $lhs = "YEAR($fieldexpr)";

    $snippet = $lhs . " IN ($placeholder)";
    // Keys are e.g. 2010, 2011, ... 2019, 2020.
    $possibles = array_keys($this->value);

    $query->addWhereExpression(
      $this->options['group'],
      $snippet,
      [$placeholder => $possibles]
    );
  }

  /**
   * Return an array of years, keyed by the year number.
   *
   * If the 'options callback' is defined, use it to return values, otherwise
   * use the predefined (translatable) versions.
   *
   * Cache the return value in $this->valueOptions for speed.
   *
   * @return array|null
   *   The array of years.
   */
  public function getValueOptions() {
    if (isset($this->valueOptions)) {
      return $this->valueOptions;
    }

    if (isset($this->definition['options callback']) && is_callable($this->definition['options callback'])) {
      if (isset($this->definition['options arguments']) && is_array($this->definition['options arguments'])) {
        $this->valueOptions = call_user_func_array($this->definition['options callback'], $this->definition['options arguments']);
      }
      else {
        $this->valueOptions = call_user_func($this->definition['options callback']);
      }
    }
    else {
      $year = intval(date("Y"));
      for($y = $year - 10; $y <= $year + 1; $y++) {
        $this->valueOptions[$y] = ($y == $year) ? $y . '*': $y;
      }
    }

    return $this->valueOptions;
  }


}
